//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


////////////////////////////////////////////////////////////////////
// Nes Mapper #1:  MMC1
// This mapper is used on numerous U.S and Japanese games, inclusing Legend of Zelda, Metroid,
// Rad Racer, Mega Man 2, and many others.
//
// Most of this mapper was written from the excellent MMC1 document written by
// Mathew Richey.
////////////////////////////////////////////////////////////////////

#ifndef NES_MAPPER_1
#define NES_MAPPER_1

#include "../NesBaseMapper.h"


#define		BANKSIZE32	false
#define		BANKSIZE16	true

#define		BANKSIZE8	false
#define		BANKSIZE4	true

class NesMapper1 : public NesBaseMapper
{
public:	
	///////////////////////////////
	// Class Procedures
	///////////////////////////////
	NesMapper1(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU);
	~NesMapper1() { };
	void initMemory();
	void write(Uint16 addr, unsigned char data);

	// MMC1 specific methods
	void acceptData(unsigned char bytData, unsigned char bytPort);
	void transSwitchSpaces();
	void trans16to32();
	void trans32to16();


	///////////////////////////////
	// Class Data Members
	///////////////////////////////
	// Define a structure for holding cartridge status
	struct 
	{
			int	intBankSwitchOffset;	// D2 of reg 0: high or low switching? (0x8000 or 0xC000?)
			bool blnPRGBankSize;		// 16k or 32k bank switching (mapped to #defines)
			bool blnCHRBankSize;		// 4k or 8k bank switching (mapped to #defines)
	} status;
	
	// Define a structure for simulating a serial port interface
	struct
	{			
			unsigned char bytBitArray;
			unsigned char bytPtr;
	} SerialPort;
	

	int intBankMap16[2];				// Map of what 16k banks are loaded
	int intBankException;				// Used to track rare 32->16 bank switching occurence
	bool blnReg3Written;				// Boolean value for trapping peculiarty
	int intLastPort;
	Sint8 regs[4];
};

#endif 
