//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper33.cpp - Implementation of mapper #33 (Taito TC0190)
//////////////////////////////////////////////////////////////////////

#include "NesMapper033.h"



NesMapper33::NesMapper33(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *&linkPPU) {
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	// Replace the standard PPU memory object with a TC0190 specific one	
	NesPPU *newPPU = new TC0190PPU();	
	newPPU->Init();
	newPPU->VRAM->mbytMirroringType = linkPPU->VRAM->mbytMirroringType;
	newPPU->VRAM->link(this);	
	delete mPpu;
	mPpu = newPPU;
	linkPPU = mPpu;
	mCpu->linkCPU(mPpu, mCpu->mApu, mCpu->mController1, mCpu->mController2, this);


	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;

	blnIrqEnable = false;

	initMaps();	
}


void NesMapper33::initMemory() {	
	if (bytPRGCount > 0) {
		swapPRGBank16(0x8000, 0);			  // First 16k bank to $8000
		swapPRGBank16(0xC000, bytPRGCount-1); // Last 16k bank to $C000
	}
}

// This is all pretty straight forward
void NesMapper33::write(Uint16 addr, unsigned char data) {		
	switch (addr) {
	case 0x8000:		
		swapPRGBank8(0x8000, data % (bytPRGCount << 1) );
		break;

	case 0x8001:
		swapPRGBank8(0xA000, data % (bytPRGCount << 1) );
		break;

	case 0x8002:	
		swapCHRBank2(0x0000, data % (bytCHRCount << 2));
		break;
	
	case 0x8003:
		swapCHRBank2(0x0800, data % (bytCHRCount << 2));
		break;
	
	case 0xA000:
		swapCHRBank1(0x1000, data % (bytCHRCount << 3));
		break;
	
	case 0xA001:
		swapCHRBank1(0x1400, data % (bytCHRCount << 3));
		break;
	
	case 0xA002:
		swapCHRBank1(0x1800, data % (bytCHRCount << 3));
		break;
	
	case 0xA003:
		swapCHRBank1(0x1C00, data % (bytCHRCount << 3));
		break;
	
	case 0xC000:
		intIrqLatch = data;
		break;
	
	case 0xC001:
		intIrqCount = intIrqLatch;
		break;
	
	case 0xC002:		
		blnIrqEnable = true;
		break;
	
	case 0xC003:		
		blnIrqEnable = false;
		break;
	
	case 0xE000:
		// Reserved (?)
		break;
	}
}