//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper34.cpp - Implementation of mapper #32 (Nina-1)
//////////////////////////////////////////////////////////////////////

#include "NesMapper034.h"



NesMapper34::NesMapper34(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU) {
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();
}


void NesMapper34::initMemory() {	
	if (bytPRGCount > 0) {
		swapPRGBank32(0);	// First 32k bank to $8000		
	}
	if (bytCHRCount > 0) {
		//swapCHRBank8(0);	// If CHRROM exists, swap in first 8k
	}
}

// This is apparently a combination of two mappers
void NesMapper34::write(Uint16 addr, unsigned char data) {		
	switch (addr) {
	case 0x7FFD:	
		swapPRGBank32(data % (bytPRGCount >> 1));
		break;
	
	case 0x7FFE:
		swapCHRBank4(0x0000, data % (bytCHRCount << 1));
		break;
	
	case 0x7FFF:
		swapCHRBank4(0x1000, data % (bytCHRCount << 1));
		break;	
	}
	
	// $8000 - $FFFF - select 32k PRGROM
	if (addr & 0x8000) {
	//if ((addr >= 0x8000) && (addr <= 0xFFFF)) {
		swapPRGBank32(data % (bytPRGCount >> 1));
	}
}