//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper065.cpp - Implementation of mapper #2 (Irem H-3001)
//////////////////////////////////////////////////////////////////////

#include "NesMapper065.h"


NesMapper65::NesMapper65(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *&linkPPU) {
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;

	// Replace the standard PPU memory object with a VRC4 specific one	
	NesPPU *newPPU = new IremHPPU();
	newPPU->Init();
	newPPU->VRAM->mbytMirroringType = linkPPU->VRAM->mbytMirroringType;
	newPPU->VRAM->link(this);	
	delete mPpu;
	mPpu = newPPU;
	linkPPU = mPpu;
	mCpu->linkCPU(mPpu, mCpu->mApu, mCpu->mController1, mCpu->mController2, this);
	
	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();

	blnIrqEnable = false;
	intIrqCount = intIrqLatch = 0;
}


void NesMapper65::initMemory() {	
	if (bytPRGCount > 0) {
		swapPRGBank16(0x8000, 0);			  // First 16k bank to $8000
		swapPRGBank16(0xC000, bytPRGCount-1); // Last 16k bank to $C000
	}
}



void NesMapper65::write(Uint16 addr, unsigned char data) {	
	switch (addr) {
	case 0x8000:
		swapPRGBank8(0x8000, data % (bytPRGCount << 1) );
		break;
	
	case 0x9000:
		mPpu->VRAM->mbytMirroringType = (data & 0x40) ? VERTICAL : HORIZONTAL;
		break;

	case 0x9001:
		mPpu->VRAM->mbytMirroringType = (data & 0x80) ? HORIZONTAL : VERTICAL;
		break;

	case 0x9003:
		blnIrqEnable = (data & 0x80) ? true : false;
		break;
	
	case 0x9004:
		intIrqCount = intIrqLatch;
		break;

	case 0x9005:	// High byte of IRQ counter
		intIrqLatch = (intIrqLatch & 0x00FF) | (data << 8);
		break;
	
	case 0x9006:	// Low byte of IRQ counter
		intIrqLatch = (intIrqLatch & 0xFF00) | data;
		break;
	
	case 0xA000:
		swapPRGBank8(0xA000, data % (bytPRGCount << 1) );
		break;
	
	case 0xB000:
		swapCHRBank1(0x0000, data % (bytCHRCount << 3) );
		break;
	case 0xB001:
		swapCHRBank1(0x0400, data % (bytCHRCount << 3) );
		break;
	case 0xB002:
		swapCHRBank1(0x0800, data % (bytCHRCount << 3) );
		break;
	case 0xB003:
		swapCHRBank1(0x0C00, data % (bytCHRCount << 3) );
		break;
	case 0xB004:
		swapCHRBank1(0x1000, data % (bytCHRCount << 3) );
		break;
	case 0xB005:
		swapCHRBank1(0x1400, data % (bytCHRCount << 3) );
		break;
	case 0xB006:
		swapCHRBank1(0x1800, data % (bytCHRCount << 3) );
		break;
	case 0xB007:
		swapCHRBank1(0x1C00, data % (bytCHRCount << 3) );
		break;

	case 0xC000:
		swapPRGBank8(0xC000, data % (bytPRGCount << 1) );
		break;


	}
}