//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper071.cpp - Implementation of mapper #71 (Camerica)
//////////////////////////////////////////////////////////////////////

#include "NesMapper071.h"


NesMapper71::NesMapper71(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU) {
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();
}


void NesMapper71::initMemory() {	
	if (bytPRGCount > 0) {
		swapPRGBank16(0x8000, 0);			  // First 16k bank to $8000
		swapPRGBank16(0xC000, bytPRGCount-1); // Last 16k bank to $C000
	}
}


void NesMapper71::write(Uint16 addr, unsigned char data) {	
	switch (addr & 0xF000) {
	case 0x9000:	// Mirroring
		mPpu->VRAM->mbytMirroringType = SINGLESCREEN;
		mPpu->VRAM->mintSSMirrorLocation = (data & 0x10) ? 0x2400 : 0x2000;
		break;
	
	case 0xC000:	// 16K PRGROM select
	case 0xD000:
	case 0xE000:
	case 0xF000:
		swapPRGBank16(0x8000, data % bytPRGCount);
		break;
	}
}