//////////////////////////////////////////////////////////////////////
// Nes Mapper 7.cpp - Implementation of mapper #7 (AOROM)
//////////////////////////////////////////////////////////////////////

#include "NesMapper7.h"


NesMapper7::NesMapper7(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU)
{
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;

	// Mapper #7, apparently all games with this mapper use single screen mapping
	mPpu->VRAM->mbytMirroringType = SINGLESCREEN;		

	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();
}


NesMapper7::~NesMapper7()
{
}

// TODO: when game is reset- call mapper initMemory function
// Initial Memory Loader
// For mapper #7, simply load the first 32K ROM bank ala mapper #0
// but also set the BankMap properly
void NesMapper7::initMemory()
{
	int intByteCtr = 0;
	if (bytPRGCount > 0) swapPRGBank32(0);
	
	// Save the fact that the first 32k bank is loaded currently
	intBankMap32 = 0;
}


void NesMapper7::write(unsigned __int16 addr, unsigned char data)
{	
	// Figure out which 32k bank the program wants, "data" is interpreted in 32k increments
	//int intPage = ((data & 0xF) % ((bytPRGCount+1) >> 1)) << 1;
	int intPage = ((data & 0xF) % (bytPRGCount >> 1));
		
	// Only swap PRG-ROM if needed
	if (intPage != intBankMap32)
	{		
		swapPRGBank32(intPage);
		intBankMap32 = intPage;
	}

	// bit 5 controls the single screen mirroring location
	if (data & 0x10) 
	{
		mPpu->VRAM->mintSSMirrorLocation = 0x2400;
	} else {		
		mPpu->VRAM->mintSSMirrorLocation = 0x2000;
	}
}
