//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// Nes PPU Memory.h: interface for the NesPPUMemory class.
//////////////////////////////////////////////////////////////////////
// Class Description:  This class takes care of all the memory mirroring,
// allocation, deletion, and quirks that make the NES VRAM so much fun.
// As an aside, the VRAM gets its own class and the CPU does not because
// I thought it would be too inefficent for the CPU memory to be contained
// in a separate object because it is fetched constantly when executing
// instructions.  While graphics are fetched constantly here, I think
// it is still pretty efficient.
//////////////////////////////////////////////////////////////////////

#ifndef NES_PPU_MEMORY
#define NES_PPU_MEMORY

#include "Porting/basetypes.h"

// Constant for various mirroring schemes (mapped to data member mbytMirroringType)
#define		HORIZONTAL				0
#define		VERTICAL				1
#define		FOURSCREEN				2
#define		SINGLESCREEN			3

class NesBaseMapper;

class NesPPUMemory
{
public:
	///////////////////////////////
	// Class Procedures
	///////////////////////////////
	NesPPUMemory();
	virtual ~NesPPUMemory();
	void init();	
	void link(NesBaseMapper *linkMapper) { mMapper = linkMapper; }
	virtual Uint8 read(Uint16 &addr, bool blnCPURead);
	Uint8 read(Uint16 addr);
	virtual void write(Uint16 &addr, Uint8 data, bool blnCPURead);
	void write(Uint16 addr, Uint8 data);
	void saveState(FILE *stream);
	void loadState(FILE *stream);
	
	///////////////////////////////
	// Class Data Members
	///////////////////////////////	
								// These three arrays make up the entire memory
	Uint8 *memory;				// Main PPU memory, used for pattern tables
	Uint8 **bytNameTable;		// Name table memory (memory for 2 name tables)
	Uint8 *palette;				// Palette memory 

	int mintIncrement;				// Amount to increment on read/write (1 or 32)
	bool mblnFirstRead;				// Flag for buffered VRAM quirk
	unsigned char mbytReadBuffer;	// Buffer for VRAM quirk
	Uint8 mbytMirroringType;			// Flag that designates what type of mirroring to use
	int mintSSMirrorLocation;		// Flag the indicates where to mirror in single-screen mode	
	int mintLastReadAddr;			// Last address read from PPU (quasi-PPU bus value)

	// Various latch values used by mappers (e.g. Punch Out!!)
	int	mintLatchValue1;
	int	mintLatchValue2;
	int mintLatchSelect1;
	int mintLatchSelect2;
	int mintLatchSelect3;
	int mintLatchSelect4;
	int mintLatchValue3;
	int mintLatchValue4;

	NesBaseMapper *mMapper;
};

#endif 
